// OpenUPSLibTest.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <conio.h>
#include "OpenUPSLib.h"

int _tmain(int argc, _TCHAR* argv[]) {
	HANDLE h = GetStdHandle(STD_OUTPUT_HANDLE);
	WORD wOldColorAttrs;
	CONSOLE_SCREEN_BUFFER_INFO csbiInfo;
	
	SetConsoleTextAttribute(h, FOREGROUND_GREEN | FOREGROUND_INTENSITY);
	GetConsoleScreenBufferInfo(h, &csbiInfo);
	wOldColorAttrs = csbiInfo.wAttributes;

	printf("Waiting 5 seconds... ");
	Sleep(5000);
	printf("OK\n");

	int lastState = 0, lastSavedFileID = 0, isCriticalWarning = 0, saveToFile = 0, wasCharging = 0;
	float cells = 6.0, cellMin = 3.1;
											
	if (UPSOpenDevice(1000)) {
		Sleep(500);
		
		printf("FW=%02d.%02d\n", getUPSVerMajor(), getUPSVerMinor());		
		printf("State=%d\n", getUPSState());
		
		int state, counter;
		counter = 0;
		while (!kbhit()) {
			state = getUPSState();
			
			if (state == 0) {
				printf("No device\n");
			}
			else {
				counter++;
				saveToFile = (counter - lastSavedFileID >= 10 ? 1 : 0);
				
				float UPSVIN = getUPSVIN(), UPSCIn = getUPSCIn(), UPSVBat = getUPSVBat(), UPSVOut = getUPSVOut(), 
					UPSVCell0 = getUPSVCell(0), UPSVCell1 = getUPSVCell(1), UPSVCell2 = getUPSVCell(2), 
					UPSVCell3 = getUPSVCell(3), UPSVCell4 = getUPSVCell(4), UPSVCell5 = getUPSVCell(5), 					
					UPSCCharge = getUPSCCharge(), UPSCDischarge = getUPSCDischarge(), 
					UPSOutputPower = getUPSOutputPower(), UPSTemperature = getUPSTemperature();
				// Check for critical values (cells and temperature)
				if ((UPSVCell0 >= 0.2 && UPSVCell0 <= cellMin) || 
						(UPSVCell1 >= 0.2 && UPSVCell1 <= cellMin) || 
						(UPSVCell2 >= 0.2 && UPSVCell2 <= cellMin) ||
						(UPSVCell3 >= 0.2 && UPSVCell3 <= cellMin) || 
						(UPSVCell4 >= 0.2 && UPSVCell4 <= cellMin) || 
						(UPSVCell5 >= 0.2 && UPSVCell5 <= cellMin) ||
						(UPSTemperature >= 60.0)) { 
					SetConsoleTextAttribute(h, FOREGROUND_RED | FOREGROUND_INTENSITY);
					isCriticalWarning = 1;
					saveToFile = 1;
				}
				else {
					isCriticalWarning = 0;
				}
				// How many cells?
				cells = (UPSVCell0 >= 0.2 ? 1 : 0) + (UPSVCell1 >= 0.2 ? 1 : 0) +
								(UPSVCell2 >= 0.2 ? 1 : 0) + (UPSVCell3 >= 0.2 ? 1 : 0) +
								(UPSVCell4 >= 0.2 ? 1 : 0) + (UPSVCell5 >= 0.2 ? 1 : 0);
				// 4.19   4.2  4.19    .
				//    12.57  3S  100%
				float pc = (UPSVBat - cellMin * cells) * 100.0 / (cells * (4.19 - cellMin));
				if (pc > 100.0) { pc = 100.0; }
				if (pc < 0.0) { pc = 0.0; }

				if (lastState != state) {
					saveToFile = 1; lastState = state;
				}

				// Charging status changed, show it
				if (state == 2) {
					if (UPSCCharge >= 0.01) {
						if (wasCharging == 0) { saveToFile = 1; }
					}
					else {
						if (wasCharging == 1) { saveToFile = 1; }
					}
				}
				
				// Save to file and show at screen
				if (saveToFile == 1) {
					switch (state) {
						case 1: 
							if (isCriticalWarning == 0)
							SetConsoleTextAttribute(h, FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_INTENSITY);
							printf("%d B ", counter); wasCharging = 0; break;	// Battery							
						case 2: 
							if (isCriticalWarning == 0) {
								if (UPSCCharge >= 0.01) {
									SetConsoleTextAttribute(h, FOREGROUND_BLUE | FOREGROUND_INTENSITY);
									wasCharging = 1;
								}
								else {
									SetConsoleTextAttribute(h, FOREGROUND_GREEN | FOREGROUND_INTENSITY);
									wasCharging = 0;
								}
							}
							printf("%d V ", counter); break;	// Vin
						case 3: 
							SetConsoleTextAttribute(h, FOREGROUND_BLUE | FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_INTENSITY);
							printf("%d U ", counter); wasCharging = 0; break;	// USB
						default: 
							SetConsoleTextAttribute(h, FOREGROUND_RED | FOREGROUND_INTENSITY);
							printf("%d E ", counter); break;	// Error
					}
									
					printf("%.2f %.2f %.1f %.2f %.2f,%.2f,%.2f,%.2f,%.2f,%.2f %.2f %.2f %.2f %.0f %.0f\n",
						UPSVIN, UPSVBat, pc, UPSVOut, UPSVCell0, UPSVCell1, UPSVCell2, UPSVCell3, UPSVCell4, UPSVCell5,
						UPSCIn, UPSCCharge, UPSCDischarge, UPSOutputPower, UPSTemperature);

					try {									
						FILE *out;
						out = fopen("OpenUPSMonitor.log", "w");
						fprintf(out, "%d ", counter);

						switch (state) {
							case 1: fprintf(out, "B "); break;	// Battery							
							case 2: fprintf(out, "V "); break;	// Vin
							case 3: fprintf(out, "U "); break;	// USB
							default: fprintf(out, "E "); break;	// Error
						}
						fprintf(out, "%.2f %.2f %.1f %.2f %.2f,%.2f,%.2f,%.2f,%.2f,%.2f %.2f %.2f %.2f %.0f %.0f\r\n",
							UPSVIN, UPSVBat, pc, UPSVOut, UPSVCell0, UPSVCell1, UPSVCell2, UPSVCell3, UPSVCell4, UPSVCell5,
							UPSCIn, UPSCCharge, UPSCDischarge, UPSOutputPower, UPSTemperature);
						fflush(out);
						fclose(out);
						lastSavedFileID = counter;
					} 
					catch(int a) {					
						printf("Error: %d\n", a); 
						lastSavedFileID = 0;
					}
				}
			}
			Sleep(1000);
		}
		UPSCloseDevice();
	}
	else {
		printf("UPS not found!\n");
		
		// Wait for user input
		_getch();
	}
	return 0;
}